from pathlib import Path
from typing import List

from pydantic import BaseModel
from strictyaml import YAML, load

# import recommendation_model
import model 

# Project Directories
PACKAGE_ROOT = Path(model.__file__).resolve().parent
ROOT = PACKAGE_ROOT.parent
CONFIG_FILE_PATH = PACKAGE_ROOT / "config.yml"
DATASET_DIR = PACKAGE_ROOT / "datasets"
TRAINED_MODEL_DIR = PACKAGE_ROOT / "trained_models"
TRANSFORMED_DATASET_DIR = PACKAGE_ROOT / "transformed_datasets"


class AppConfig(BaseModel):
    """
    Application-level config.
    """

    package_name: str
    training_movies_file: str
    training_credits_file: str
    transformed_fie: str
    pipeline_save_file: str


class ModelConfig(BaseModel):
    """
    All configuration relevant to model
    training and feature engineering.
    """

    max_features: int
    stop_words: str
    variable_to_merge: str
    variables_to_keep: List[str]
    variables_to_convert: List[str]
    variables_top_three: str
    variables_to_fetch_director: str
    variables_to_collapse: List[str]
    variables_to_split: str
    variables_to_create_tag: List[str]
    variables_to_drop: List[str]
    variable_new: str


class Config(BaseModel):
    """Master config object."""

    app_conf: AppConfig
    model_conf: ModelConfig


def find_config_file() -> Path:
    """Locate the configuration file."""
    if CONFIG_FILE_PATH.is_file():
        return CONFIG_FILE_PATH
    raise Exception(f"Config not found at {CONFIG_FILE_PATH!r}")


def fetch_config_from_yaml(cfg_path: Path = None) -> YAML:
    """Parse YAML containing the package configuration."""

    if not cfg_path:
        cfg_path = find_config_file()

    if cfg_path:
        with open(cfg_path, "r") as conf_file:
            parsed_config = load(conf_file.read())
            return parsed_config
    raise OSError(f"Did not find config file at path: {cfg_path}")


def create_and_validate_config(parsed_config: YAML = None) -> Config:
    """Run validation on config values."""
    if parsed_config is None:
        parsed_config = fetch_config_from_yaml()

    # specify the data attribute from the strictyaml YAML type.
    _config = Config(
        app_conf=AppConfig(**parsed_config.data),
        model_conf=ModelConfig(**parsed_config.data),
    )

    return _config


config = create_and_validate_config()